@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Zend Engine                                                          |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1998-2018 Zend Technologies Ltd. (http://www.zend.com) |
+   | Copyright (c) Zend Technologies Ltd. (http://www.zend.com)           |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.00 of the Zend license,     |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -29,9 +29,12 @@
 /* Protection from recursive self-referencing class constants */
 #define IS_CONSTANT_VISITED_MARK    0x80
 
-#define IS_CONSTANT_VISITED(zv)     (Z_ACCESS_FLAGS_P(zv) & IS_CONSTANT_VISITED_MARK)
-#define MARK_CONSTANT_VISITED(zv)   Z_ACCESS_FLAGS_P(zv) |= IS_CONSTANT_VISITED_MARK
-#define RESET_CONSTANT_VISITED(zv)  Z_ACCESS_FLAGS_P(zv) &= ~IS_CONSTANT_VISITED_MARK
+#define IS_CONSTANT_VISITED(zv)     (Z_CONSTANT_FLAGS_P(zv) & IS_CONSTANT_VISITED_MARK)
+#define MARK_CONSTANT_VISITED(zv)   Z_CONSTANT_FLAGS_P(zv) |= IS_CONSTANT_VISITED_MARK
+#define RESET_CONSTANT_VISITED(zv)  Z_CONSTANT_FLAGS_P(zv) &= ~IS_CONSTANT_VISITED_MARK
+
+/* Use for special null/true/false constants. */
+static zend_constant *null_const, *true_const, *false_const;
 
 void free_zend_constant(zval *zv)
 {
@@ -83,9 +86,9 @@ static int clean_module_constant(zval *el, void *arg)
 	int module_number = *(int *)arg;
 
 	if (ZEND_CONSTANT_MODULE_NUMBER(c) == module_number) {
-		return 1;
+		return ZEND_HASH_APPLY_REMOVE;
 	} else {
-		return 0;
+		return ZEND_HASH_APPLY_KEEP;
 	}
 }
 
@@ -95,13 +98,10 @@ void clean_module_constants(int module_number)
 	zend_hash_apply_with_argument(EG(zend_constants), clean_module_constant, (void *) &module_number);
 }
 
-
-int zend_startup_constants(void)
+void zend_startup_constants(void)
 {
 	EG(zend_constants) = (HashTable *) malloc(sizeof(HashTable));
-
 	zend_hash_init(EG(zend_constants), 128, NULL, ZEND_CONSTANT_DTOR, 1);
-	return SUCCESS;
 }
 
 
@@ -128,22 +128,24 @@ void zend_register_standard_constants(void)
 
 	REGISTER_MAIN_LONG_CONSTANT("DEBUG_BACKTRACE_PROVIDE_OBJECT", DEBUG_BACKTRACE_PROVIDE_OBJECT, CONST_PERSISTENT | CONST_CS);
 	REGISTER_MAIN_LONG_CONSTANT("DEBUG_BACKTRACE_IGNORE_ARGS", DEBUG_BACKTRACE_IGNORE_ARGS, CONST_PERSISTENT | CONST_CS);
-	/* true/false constants */
-	{
-		REGISTER_MAIN_BOOL_CONSTANT("TRUE", 1, CONST_PERSISTENT | CONST_CT_SUBST);
-		REGISTER_MAIN_BOOL_CONSTANT("FALSE", 0, CONST_PERSISTENT | CONST_CT_SUBST);
-		REGISTER_MAIN_BOOL_CONSTANT("ZEND_THREAD_SAFE", ZTS_V, CONST_PERSISTENT | CONST_CS);
-		REGISTER_MAIN_BOOL_CONSTANT("ZEND_DEBUG_BUILD", ZEND_DEBUG, CONST_PERSISTENT | CONST_CS);
-	}
-	REGISTER_MAIN_NULL_CONSTANT("NULL", CONST_PERSISTENT | CONST_CT_SUBST);
+	REGISTER_MAIN_BOOL_CONSTANT("ZEND_THREAD_SAFE", ZTS_V, CONST_PERSISTENT | CONST_CS);
+	REGISTER_MAIN_BOOL_CONSTANT("ZEND_DEBUG_BUILD", ZEND_DEBUG, CONST_PERSISTENT | CONST_CS);
+
+	/* Special constants true/false/null.  */
+	REGISTER_MAIN_BOOL_CONSTANT("TRUE", 1, CONST_PERSISTENT);
+	REGISTER_MAIN_BOOL_CONSTANT("FALSE", 0, CONST_PERSISTENT);
+	REGISTER_MAIN_NULL_CONSTANT("NULL", CONST_PERSISTENT);
+
+	true_const = zend_hash_str_find_ptr(EG(zend_constants), "TRUE", sizeof("TRUE")-1);
+	false_const = zend_hash_str_find_ptr(EG(zend_constants), "FALSE", sizeof("FALSE")-1);
+	null_const = zend_hash_str_find_ptr(EG(zend_constants), "NULL", sizeof("NULL")-1);
 }
 
 
-int zend_shutdown_constants(void)
+void zend_shutdown_constants(void)
 {
 	zend_hash_destroy(EG(zend_constants));
 	free(EG(zend_constants));
-	return SUCCESS;
 }
 
 ZEND_API void zend_register_null_constant(const char *name, size_t name_len, int flags, int module_number)
@@ -156,7 +158,7 @@ ZEND_API void zend_register_null_constant(const char *name, size_t name_len, int
 	zend_register_constant(&c);
 }
 
-ZEND_API void zend_register_bool_constant(const char *name, size_t name_len, zend_bool bval, int flags, int module_number)
+ZEND_API void zend_register_bool_constant(const char *name, size_t name_len, bool bval, int flags, int module_number)
 {
 	zend_constant c;
 
@@ -188,7 +190,7 @@ ZEND_API void zend_register_double_constant(const char *name, size_t name_len, d
 }
 
 
-ZEND_API void zend_register_stringl_constant(const char *name, size_t name_len, char *strval, size_t strlen, int flags, int module_number)
+ZEND_API void zend_register_stringl_constant(const char *name, size_t name_len, const char *strval, size_t strlen, int flags, int module_number)
 {
 	zend_constant c;
 
@@ -199,12 +201,12 @@ ZEND_API void zend_register_stringl_constant(const char *name, size_t name_len,
 }
 
 
-ZEND_API void zend_register_string_constant(const char *name, size_t name_len, char *strval, int flags, int module_number)
+ZEND_API void zend_register_string_constant(const char *name, size_t name_len, const char *strval, int flags, int module_number)
 {
 	zend_register_stringl_constant(name, name_len, strval, strlen(strval), flags, module_number);
 }
 
-static zend_constant *zend_get_special_constant(const char *name, size_t name_len)
+static zend_constant *zend_get_halt_offset_constant(const char *name, size_t name_len)
 {
 	zend_constant *c;
 	static const char haltoff[] = "__COMPILER_HALT_OFFSET__";
@@ -230,100 +232,179 @@ static zend_constant *zend_get_special_constant(const char *name, size_t name_le
 	}
 }
 
-ZEND_API int zend_verify_const_access(zend_class_constant *c, zend_class_entry *scope) /* {{{ */
+ZEND_API zend_constant *_zend_get_special_const(const char *name, size_t len) /* {{{ */
 {
-	if (Z_ACCESS_FLAGS(c->value) & ZEND_ACC_PUBLIC) {
+	if (len == 4) {
+		if ((name[0] == 'n' || name[0] == 'N') &&
+			(name[1] == 'u' || name[1] == 'U') &&
+			(name[2] == 'l' || name[2] == 'L') &&
+			(name[3] == 'l' || name[3] == 'L')
+		) {
+			return null_const;
+		}
+		if ((name[0] == 't' || name[0] == 'T') &&
+			(name[1] == 'r' || name[1] == 'R') &&
+			(name[2] == 'u' || name[2] == 'U') &&
+			(name[3] == 'e' || name[3] == 'E')
+		) {
+			return true_const;
+		}
+	} else {
+		if ((name[0] == 'f' || name[0] == 'F') &&
+			(name[1] == 'a' || name[1] == 'A') &&
+			(name[2] == 'l' || name[2] == 'L') &&
+			(name[3] == 's' || name[3] == 'S') &&
+			(name[4] == 'e' || name[4] == 'E')
+		) {
+			return false_const;
+		}
+	}
+	return NULL;
+}
+/* }}} */
+
+ZEND_API bool zend_verify_const_access(zend_class_constant *c, zend_class_entry *scope) /* {{{ */
+{
+	if (ZEND_CLASS_CONST_FLAGS(c) & ZEND_ACC_PUBLIC) {
 		return 1;
-	} else if (Z_ACCESS_FLAGS(c->value) & ZEND_ACC_PRIVATE) {
+	} else if (ZEND_CLASS_CONST_FLAGS(c) & ZEND_ACC_PRIVATE) {
 		return (c->ce == scope);
 	} else {
-		ZEND_ASSERT(Z_ACCESS_FLAGS(c->value) & ZEND_ACC_PROTECTED);
+		ZEND_ASSERT(ZEND_CLASS_CONST_FLAGS(c) & ZEND_ACC_PROTECTED);
 		return zend_check_protected(c->ce, scope);
 	}
 }
 /* }}} */
 
-static inline zend_constant *zend_get_constant_str_impl(const char *name, size_t name_len)
+static zend_constant *zend_get_constant_str_impl(const char *name, size_t name_len)
 {
-	zend_constant *c;
-	ALLOCA_FLAG(use_heap)
-
-	if ((c = zend_hash_str_find_ptr(EG(zend_constants), name, name_len)) == NULL) {
-		char *lcname = do_alloca(name_len + 1, use_heap);
-		zend_str_tolower_copy(lcname, name, name_len);
-		if ((c = zend_hash_str_find_ptr(EG(zend_constants), lcname, name_len)) != NULL) {
-			if (ZEND_CONSTANT_FLAGS(c) & CONST_CS) {
-				c = NULL;
-			}
-		} else {
-			c = zend_get_special_constant(name, name_len);
-		}
-		free_alloca(lcname, use_heap);
+	zend_constant *c = zend_hash_str_find_ptr(EG(zend_constants), name, name_len);
+	if (c) {
+		return c;
 	}
 
-	return c;
+	c = zend_get_halt_offset_constant(name, name_len);
+	if (c) {
+		return c;
+	}
+
+	return zend_get_special_const(name, name_len);
 }
 
 ZEND_API zval *zend_get_constant_str(const char *name, size_t name_len)
 {
 	zend_constant *c = zend_get_constant_str_impl(name, name_len);
-	return c ? &c->value : NULL;
+	if (c) {
+		return &c->value;
+	}
+	return NULL;
 }
 
-static inline zend_constant *zend_get_constant_impl(zend_string *name)
+static zend_constant *zend_get_constant_impl(zend_string *name)
 {
-    zval *zv;
-	zend_constant *c;
-	ALLOCA_FLAG(use_heap)
-
-	zv = zend_hash_find(EG(zend_constants), name);
-	if (zv == NULL) {
-		char *lcname = do_alloca(ZSTR_LEN(name) + 1, use_heap);
-		zend_str_tolower_copy(lcname, ZSTR_VAL(name), ZSTR_LEN(name));
-		zv = zend_hash_str_find(EG(zend_constants), lcname, ZSTR_LEN(name));
-		if (zv != NULL) {
-			c = Z_PTR_P(zv);
-			if (ZEND_CONSTANT_FLAGS(c) & CONST_CS) {
-				c = NULL;
-			}
-		} else {
-			c = zend_get_special_constant(ZSTR_VAL(name), ZSTR_LEN(name));
-		}
-		free_alloca(lcname, use_heap);
+	zend_constant *c = zend_hash_find_ptr(EG(zend_constants), name);
+	if (c) {
 		return c;
-	} else {
-		return (zend_constant *) Z_PTR_P(zv);
 	}
+
+	c = zend_get_halt_offset_constant(ZSTR_VAL(name), ZSTR_LEN(name));
+	if (c) {
+		return c;
+	}
+
+	return zend_get_special_const(ZSTR_VAL(name), ZSTR_LEN(name));
 }
 
 ZEND_API zval *zend_get_constant(zend_string *name)
 {
 	zend_constant *c = zend_get_constant_impl(name);
-	return c ? &c->value : NULL;
+	if (c) {
+		return &c->value;
+	}
+	return NULL;
 }
 
-static zend_bool is_access_deprecated(const zend_constant *c, const char *access_name) {
-	const char *ns_sep = zend_memrchr(ZSTR_VAL(c->name), '\\', ZSTR_LEN(c->name));
-	if (ns_sep) {
-		/* Namespaces are always case-insensitive. Only compare shortname. */
-		size_t shortname_offset = ns_sep - ZSTR_VAL(c->name) + 1;
-		size_t shortname_len = ZSTR_LEN(c->name) - shortname_offset;
-		return memcmp(
-			access_name + shortname_offset,
-			ZSTR_VAL(c->name) + shortname_offset,
-			shortname_len
-		) != 0;
+ZEND_API zval *zend_get_class_constant_ex(zend_string *class_name, zend_string *constant_name, zend_class_entry *scope, uint32_t flags)
+{
+	zend_class_entry *ce = NULL;
+	zend_class_constant *c = NULL;
+	zval *ret_constant = NULL;
+
+	if (ZSTR_HAS_CE_CACHE(class_name)) {
+		ce = ZSTR_GET_CE_CACHE(class_name);
+		if (!ce) {
+			ce = zend_fetch_class(class_name, flags);
+		}
+	} else if (zend_string_equals_literal_ci(class_name, "self")) {
+		if (UNEXPECTED(!scope)) {
+			zend_throw_error(NULL, "Cannot access \"self\" when no class scope is active");
+			goto failure;
+		}
+		ce = scope;
+	} else if (zend_string_equals_literal_ci(class_name, "parent")) {
+		if (UNEXPECTED(!scope)) {
+			zend_throw_error(NULL, "Cannot access \"parent\" when no class scope is active");
+			goto failure;
+		} else if (UNEXPECTED(!scope->parent)) {
+			zend_throw_error(NULL, "Cannot access \"parent\" when current class scope has no parent");
+			goto failure;
+		} else {
+			ce = scope->parent;
+		}
+	} else if (zend_string_equals_literal_ci(class_name, "static")) {
+		ce = zend_get_called_scope(EG(current_execute_data));
+		if (UNEXPECTED(!ce)) {
+			zend_throw_error(NULL, "Cannot access \"static\" when no class scope is active");
+			goto failure;
+		}
 	} else {
-		/* No namespace, compare whole name */
-		return memcmp(access_name, ZSTR_VAL(c->name), ZSTR_LEN(c->name)) != 0;
+		ce = zend_fetch_class(class_name, flags);
 	}
+	if (ce) {
+		c = zend_hash_find_ptr(CE_CONSTANTS_TABLE(ce), constant_name);
+		if (c == NULL) {
+			if ((flags & ZEND_FETCH_CLASS_SILENT) == 0) {
+				zend_throw_error(NULL, "Undefined constant %s::%s", ZSTR_VAL(class_name), ZSTR_VAL(constant_name));
+				goto failure;
+			}
+			ret_constant = NULL;
+		} else {
+			if (!zend_verify_const_access(c, scope)) {
+				if ((flags & ZEND_FETCH_CLASS_SILENT) == 0) {
+					zend_throw_error(NULL, "Cannot access %s constant %s::%s", zend_visibility_string(ZEND_CLASS_CONST_FLAGS(c)), ZSTR_VAL(class_name), ZSTR_VAL(constant_name));
+				}
+				goto failure;
+			}
+			ret_constant = &c->value;
+		}
+	}
+
+	if (ret_constant && Z_TYPE_P(ret_constant) == IS_CONSTANT_AST) {
+		zend_result ret;
+
+		if (IS_CONSTANT_VISITED(ret_constant)) {
+			zend_throw_error(NULL, "Cannot declare self-referencing constant %s::%s", ZSTR_VAL(class_name), ZSTR_VAL(constant_name));
+			ret_constant = NULL;
+			goto failure;
+		}
+
+		MARK_CONSTANT_VISITED(ret_constant);
+		ret = zval_update_constant_ex(ret_constant, c->ce);
+		RESET_CONSTANT_VISITED(ret_constant);
+
+		if (UNEXPECTED(ret != SUCCESS)) {
+			ret_constant = NULL;
+			goto failure;
+		}
+	}
+failure:
+	return ret_constant;
 }
 
 ZEND_API zval *zend_get_constant_ex(zend_string *cname, zend_class_entry *scope, uint32_t flags)
 {
 	zend_constant *c;
 	const char *colon;
-	zend_class_entry *ce = NULL;
 	const char *name = ZSTR_VAL(cname);
 	size_t name_len = ZSTR_LEN(cname);
 
@@ -339,22 +420,29 @@ ZEND_API zval *zend_get_constant_ex(zend_string *cname, zend_class_entry *scope,
 		int class_name_len = colon - name - 1;
 		size_t const_name_len = name_len - class_name_len - 2;
 		zend_string *constant_name = zend_string_init(colon + 1, const_name_len, 0);
-		zend_string *class_name = zend_string_init(name, class_name_len, 0);
+		zend_string *class_name = zend_string_init_interned(name, class_name_len, 0);
+		zval *ret_constant = zend_get_class_constant_ex(class_name, constant_name, scope, flags);
+
+		zend_string_release_ex(class_name, 0);
+		zend_string_efree(constant_name);
+		return ret_constant;
+/*
+		zend_class_entry *ce = NULL;
 		zend_class_constant *c = NULL;
 		zval *ret_constant = NULL;
 
 		if (zend_string_equals_literal_ci(class_name, "self")) {
 			if (UNEXPECTED(!scope)) {
-				zend_throw_error(NULL, "Cannot access self:: when no class scope is active");
+				zend_throw_error(NULL, "Cannot access \"self\" when no class scope is active");
 				goto failure;
 			}
 			ce = scope;
 		} else if (zend_string_equals_literal_ci(class_name, "parent")) {
 			if (UNEXPECTED(!scope)) {
-				zend_throw_error(NULL, "Cannot access parent:: when no class scope is active");
+				zend_throw_error(NULL, "Cannot access \"parent\" when no class scope is active");
 				goto failure;
 			} else if (UNEXPECTED(!scope->parent)) {
-				zend_throw_error(NULL, "Cannot access parent:: when current class scope has no parent");
+				zend_throw_error(NULL, "Cannot access \"parent\" when current class scope has no parent");
 				goto failure;
 			} else {
 				ce = scope->parent;
@@ -362,24 +450,24 @@ ZEND_API zval *zend_get_constant_ex(zend_string *cname, zend_class_entry *scope,
 		} else if (zend_string_equals_literal_ci(class_name, "static")) {
 			ce = zend_get_called_scope(EG(current_execute_data));
 			if (UNEXPECTED(!ce)) {
-				zend_throw_error(NULL, "Cannot access static:: when no class scope is active");
+				zend_throw_error(NULL, "Cannot access \"static\" when no class scope is active");
 				goto failure;
 			}
 		} else {
 			ce = zend_fetch_class(class_name, flags);
 		}
 		if (ce) {
-			c = zend_hash_find_ptr(&ce->constants_table, constant_name);
+			c = zend_hash_find_ptr(CE_CONSTANTS_TABLE(ce), constant_name);
 			if (c == NULL) {
 				if ((flags & ZEND_FETCH_CLASS_SILENT) == 0) {
-					zend_throw_error(NULL, "Undefined class constant '%s::%s'", ZSTR_VAL(class_name), ZSTR_VAL(constant_name));
+					zend_throw_error(NULL, "Undefined constant %s::%s", ZSTR_VAL(class_name), ZSTR_VAL(constant_name));
 					goto failure;
 				}
 				ret_constant = NULL;
 			} else {
 				if (!zend_verify_const_access(c, scope)) {
 					if ((flags & ZEND_FETCH_CLASS_SILENT) == 0) {
-						zend_throw_error(NULL, "Cannot access %s const %s::%s", zend_visibility_string(Z_ACCESS_FLAGS(c->value)), ZSTR_VAL(class_name), ZSTR_VAL(constant_name));
+						zend_throw_error(NULL, "Cannot access %s constant %s::%s", zend_visibility_string(ZEND_CLASS_CONST_FLAGS(c)), ZSTR_VAL(class_name), ZSTR_VAL(constant_name));
 					}
 					goto failure;
 				}
@@ -388,10 +476,10 @@ ZEND_API zval *zend_get_constant_ex(zend_string *cname, zend_class_entry *scope,
 		}
 
 		if (ret_constant && Z_TYPE_P(ret_constant) == IS_CONSTANT_AST) {
-			int ret;
+			zend_result ret;
 
 			if (IS_CONSTANT_VISITED(ret_constant)) {
-				zend_throw_error(NULL, "Cannot declare self-referencing constant '%s::%s'", ZSTR_VAL(class_name), ZSTR_VAL(constant_name));
+				zend_throw_error(NULL, "Cannot declare self-referencing constant %s::%s", ZSTR_VAL(class_name), ZSTR_VAL(constant_name));
 				ret_constant = NULL;
 				goto failure;
 			}
@@ -409,6 +497,7 @@ failure:
 		zend_string_release_ex(class_name, 0);
 		zend_string_efree(constant_name);
 		return ret_constant;
+*/
 	}
 
 	/* non-class constant */
@@ -421,33 +510,22 @@ failure:
 		size_t lcname_len;
 		ALLOCA_FLAG(use_heap)
 
+		/* Lowercase the namespace portion */
 		lcname_len = prefix_len + 1 + const_name_len;
 		lcname = do_alloca(lcname_len + 1, use_heap);
 		zend_str_tolower_copy(lcname, name, prefix_len);
-		/* Check for namespace constant */
 
 		lcname[prefix_len] = '\\';
 		memcpy(lcname + prefix_len + 1, constant_name, const_name_len + 1);
 
-		if ((c = zend_hash_str_find_ptr(EG(zend_constants), lcname, lcname_len)) == NULL) {
-			/* try lowercase */
-			zend_str_tolower(lcname + prefix_len + 1, const_name_len);
-			if ((c = zend_hash_str_find_ptr(EG(zend_constants), lcname, lcname_len)) != NULL) {
-				if ((ZEND_CONSTANT_FLAGS(c) & CONST_CS) != 0) {
-					c = NULL;
-				}
-			}
-		}
+		c = zend_hash_str_find_ptr(EG(zend_constants), lcname, lcname_len);
 		free_alloca(lcname, use_heap);
 
 		if (!c) {
-			if (!(flags & IS_CONSTANT_UNQUALIFIED)) {
-				return NULL;
+			if (flags & IS_CONSTANT_UNQUALIFIED_IN_NAMESPACE) {
+				/* name requires runtime resolution, need to check non-namespaced name */
+				c = zend_get_constant_str_impl(constant_name, const_name_len);
 			}
-
-			/* name requires runtime resolution, need to check non-namespaced name */
-			c = zend_get_constant_str_impl(constant_name, const_name_len);
-			name = constant_name;
 		}
 	} else {
 		if (cname) {
@@ -458,18 +536,15 @@ failure:
 	}
 
 	if (!c) {
+		if (!(flags & ZEND_FETCH_CLASS_SILENT)) {
+			zend_throw_error(NULL, "Undefined constant \"%s\"", name);
+		}
 		return NULL;
 	}
 
-	if (!(flags & ZEND_GET_CONSTANT_NO_DEPRECATION_CHECK)) {
-		if (!(ZEND_CONSTANT_FLAGS(c) & (CONST_CS|CONST_CT_SUBST)) && is_access_deprecated(c, name)) {
-			zend_error(E_DEPRECATED,
-				"Case-insensitive constants are deprecated. "
-				"The correct casing for this constant is \"%s\"",
-				ZSTR_VAL(c->name));
-		}
+	if (!(flags & ZEND_FETCH_CLASS_SILENT) && (ZEND_CONSTANT_FLAGS(c) & CONST_DEPRECATED)) {
+		zend_error(E_DEPRECATED, "Constant %s is deprecated", name);
 	}
-
 	return &c->value;
 }
 
@@ -486,43 +561,35 @@ static void* zend_hash_add_constant(HashTable *ht, zend_string *key, zend_consta
 	return ret;
 }
 
-ZEND_API int zend_register_constant(zend_constant *c)
+ZEND_API zend_result zend_register_constant(zend_constant *c)
 {
 	zend_string *lowercase_name = NULL;
 	zend_string *name;
-	int ret = SUCCESS;
+	zend_result ret = SUCCESS;
+	bool persistent = (ZEND_CONSTANT_FLAGS(c) & CONST_PERSISTENT) != 0;
 
 #if 0
 	printf("Registering constant for module %d\n", c->module_number);
 #endif
 
-	if (!(ZEND_CONSTANT_FLAGS(c) & CONST_CS)) {
-		lowercase_name = zend_string_tolower_ex(c->name, ZEND_CONSTANT_FLAGS(c) & CONST_PERSISTENT);
+	const char *slash = strrchr(ZSTR_VAL(c->name), '\\');
+	if (slash) {
+		lowercase_name = zend_string_init(ZSTR_VAL(c->name), ZSTR_LEN(c->name), persistent);
+		zend_str_tolower(ZSTR_VAL(lowercase_name), slash - ZSTR_VAL(c->name));
 		lowercase_name = zend_new_interned_string(lowercase_name);
 		name = lowercase_name;
 	} else {
-		char *slash = strrchr(ZSTR_VAL(c->name), '\\');
-		if (slash) {
-			lowercase_name = zend_string_init(ZSTR_VAL(c->name), ZSTR_LEN(c->name), ZEND_CONSTANT_FLAGS(c) & CONST_PERSISTENT);
-			zend_str_tolower(ZSTR_VAL(lowercase_name), slash - ZSTR_VAL(c->name));
-			lowercase_name = zend_new_interned_string(lowercase_name);
-			name = lowercase_name;
-		} else {
-			name = c->name;
-		}
+		name = c->name;
 	}
 
-	/* Check if the user is trying to define the internal pseudo constant name __COMPILER_HALT_OFFSET__ */
+	/* Check if the user is trying to define any special constant */
 	if (zend_string_equals_literal(name, "__COMPILER_HALT_OFFSET__")
-		|| zend_hash_add_constant(EG(zend_constants), name, c) == NULL) {
-
-		/* The internal __COMPILER_HALT_OFFSET__ is prefixed by NULL byte */
-		if (ZSTR_VAL(c->name)[0] == '\0' && ZSTR_LEN(c->name) > sizeof("\0__COMPILER_HALT_OFFSET__")-1
-			&& memcmp(ZSTR_VAL(name), "\0__COMPILER_HALT_OFFSET__", sizeof("\0__COMPILER_HALT_OFFSET__")) == 0) {
-		}
-		zend_error(E_NOTICE,"Constant %s already defined", ZSTR_VAL(name));
+		|| (!persistent && zend_get_special_const(ZSTR_VAL(name), ZSTR_LEN(name)))
+		|| zend_hash_add_constant(EG(zend_constants), name, c) == NULL
+	) {
+		zend_error(E_WARNING, "Constant %s already defined", ZSTR_VAL(name));
 		zend_string_release(c->name);
-		if (!(ZEND_CONSTANT_FLAGS(c) & CONST_PERSISTENT)) {
+		if (!persistent) {
 			zval_ptr_dtor_nogc(&c->value);
 		}
 		ret = FAILURE;
@@ -532,14 +599,3 @@ ZEND_API int zend_register_constant(zend_constant *c)
 	}
 	return ret;
 }
-
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * indent-tabs-mode: t
- * End:
- * vim600: sw=4 ts=4 fdm=marker
- * vim<600: sw=4 ts=4
- */
